

/* aim.c -- aim in some direction while standing still */ 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <poll.h>
#include <signal.h>
#include <math.h>

#include <utilib.h>
#include "hmc6352.h"
#include "mgu.h"

/* Aim: stand, turn around until 
	facing in the desired direction. 
	For standing still, the returned 
	value of motor time is set as:
		mimpLs(dt);
		mimpRs(-dt);

	For movement for a time M (say 400 ms or so)
		mimpLs(M+dt);
		mimpRs(M-dt);
*/ 

int aim(double desh, int *pdt)
{
	double whd, ah, dth; 
	int dt;

	*pdt = 0;

	whd = rdhd();
	ah = anorm(whd);
	dth = anorm(desh-ah); 

	/* Movement -- if too much we will overshoot */
	if(dth > 0.0)
		dt = (int) (dth / 2.0) + 6;
	else
		dt = (int) (dth / 2.0) - 6;

	printf("%s(%d): Desired %G actual %G (%G) difference %G dt=%d\n", 
		__FILE__, __LINE__, 
		desh, ah, whd, dth, dt); 

	if(fabs(dth) < 2.0) return(0);
	*pdt = dt;
	return(1);
}

int aimloop(double desh)
{
	int dt, aa;
	for(;;)
	{
		aa = aim(desh, &dt); 

		/* If the difference is positive, turn right (left fwd, right reverse), 
			if the difference is negative turn left (left reverse right forward).  */

		if(aa == 0) break;

		mimpLs(dt);
		mimpRs(-dt);
	}
	return(0);
}

#if(0)

int aimloopi(double desh, int *pdt)
{
	double whd, ah, dth; 
	int dt;

	for(;;)
	{
		whd = rdhd();
		ah = anorm(whd);
		dth = anorm(desh-ah); 

		/* Movement -- if too much we will overshoot */
		if(dth > 0.0)
			dt = (int) (dth / 2.0) + 6;
		else
			dt = (int) (dth / 2.0) - 6;

		printf("%s(%d): Desired %G actual %G (%G) difference %G dt=%d\n", 
			__FILE__, __LINE__, 
			desh, ah, whd, dth, dt); 

		if(fabs(dth) < 2.0) break;

		/* If the difference is positive, turn right (left fwd, right reverse), 
			if the difference is negative turn left (left reverse right forward).  */

		mimpLs(dt);
		mimpRs(-dt);
	}

	return(0);
}
#endif


int main(int argc, char **argv)
{
	double ah, desh;
	char *stop;

	desh = 0.0;
	if(argc >= 2) desh = anorm(strtod(argv[1], &stop)); 

	aimloop(desh);

	ah = anorm(rdhd());
	printf("Heading now = %G\n", ah); 

	return(0);
}

#if(0)

void mstop();
void mimpL(int duration, int direction);
void mimpR(int duration, int direction);
void mimpLR(int duration, int direction);


void mimpLs(int duration);
void mimpRs(int duration);
void mimpLRs(int duration);



int heading()
{
	double hh;

	hh = rdhd();
	printf("Heading = %G\n", hh); 
	return(0);
}

#endif


